"""
Smart contract interfaces for the 0G Compute Network SDK.
"""

from .abis import (
    SERVING_CONTRACT_ABI,
    LEDGER_CONTRACT_ABI,
    DEFAULT_SERVING_ADDRESS,
    DEFAULT_LEDGER_ADDRESS,
    LEDGER_ADDRESS,
    INFERENCE_SERVING_ADDRESS,
    FINETUNING_SERVING_ADDRESS,
    AUTOMATA_CONTRACT_ADDRESS
)

__all__ = [
    "SERVING_CONTRACT_ABI",
    "LEDGER_CONTRACT_ABI",
    "DEFAULT_SERVING_ADDRESS",
    "DEFAULT_LEDGER_ADDRESS",
    "LEDGER_ADDRESS",
    "INFERENCE_SERVING_ADDRESS",
    "FINETUNING_SERVING_ADDRESS",
    "AUTOMATA_CONTRACT_ADDRESS",
]